@@ -508,14 +508,6 @@ func f() {
 func TestImplementations(t *testing.T) {
 	t.Parallel()
 
-	// types.CheckExpr, now used in the rangeint modernizer, had a
-	// data race (#71817) that was fixed in go1.25 and backported
-	// to go1.24 but not to go1.23. Although in principle it could
-	// affect a lot of tests, it (weirdly) only seems to show up
-	// in this one (#72082). Rather than backport again, we
-	// suppress this test.
-	testenv.NeedsGo1Point(t, 24)
-
 	tree := writeTree(t, `
 -- a.go --
 package a
@@ -1063,7 +1055,7 @@ var _ io.Reader = C{}
 
 type C struct{}
 
-// Read implements io.Reader.
+// Read implements [io.Reader].
 func (c C) Read(p []byte) (n int, err error) {
 	panic("unimplemented")
 }
